@@ -187,6 +187,17 @@ async fn handle_build_plan(
             stop_signal: None,
         }
     }
+    fn scratch_spec() -> ImageSpecification {
+        ImageSpecification {
+            architecture: buildkit_frontend::oci::Architecture::Amd64, // TODO
+            author: None,
+            config: Some(empty_image_config()),
+            created: None,
+            history: None,
+            os: buildkit_frontend::oci::OperatingSystem::Linux,
+            rootfs: None,
+        }
+    }
 
     async fn get_local_source_for_copy(
         bridge: &Bridge,
@@ -239,7 +250,31 @@ async fn handle_build_plan(
             cmd
         }
 
+        fn iter_hm_sorted<K: Ord, V>(hm: &HashMap<K, V>) -> Vec<(&K, &V)> {
+            let mut v = hm.iter().collect::<Vec<_>>();
+            v.sort_unstable_by_key(|(k, _)| *k);
+            v
+        }
+
+        fn add_envs<'a>(mut cmd: Command<'a>, envs: &HashMap<String, String>) -> Command<'a> {
+            for (k, v) in iter_hm_sorted(envs) {
+                cmd = cmd.env(k, v);
+            }
+            cmd
+        }
+
         let new_node: (OwnedOutput, Arc<ImageSpecification>) = match node {
+            /*
+                resolve_image_config will fail if we try to resolve an empty
+                image (like that produced by FROM scratch). Therefore, we have a
+                special case here for scratch images that don't try to resolve
+                its spec.
+            */
+            FromScratch { scratch_ref } => {
+                let img_s =
+                    Source::image(scratch_ref.as_ref().unwrap()).custom_name("from(\"scratch\")");
+                (img_s.ref_counted().into(), Arc::new(scratch_spec()))
+            }
             From {
                 image_ref,
                 display_name,
@@ -247,10 +282,13 @@ async fn handle_build_plan(
                 let img_s =
                     Source::image(image_ref).custom_name(format!("from({:?})", display_name));
                 let log_name = format!("from({:?}) :: resolve image config", display_name);
-                let (_, resolved_config) = bridge
-                    .resolve_image_config(&img_s, Some(&log_name))
-                    .await
-                    .expect("Resolution failed.");
+                let resolved_config =
+                    match bridge.resolve_image_config(&img_s, Some(&log_name)).await {
+                        Ok((_, x)) => x,
+                        Err(e) => {
+                            panic!("Failed to resolve image config: {:?}", e); // unreachable
+                        }
+                    };
                 (img_s.ref_counted().into(), Arc::new(resolved_config))
             }
             Run {
@@ -266,9 +304,7 @@ async fn handle_build_plan(
                 let mut cmd = new_cmd(&*parent_config, &cwd[..], &parent.0, &options)
                     .args(&["-c", &command[..]])
                     .custom_name(format!("run({:?})", command));
-                for (k, v) in additional_envs.iter() {
-                    cmd = cmd.env(k, v);
-                }
+                cmd = add_envs(cmd, additional_envs);
                 let o = OwnedOutput::from_command(cmd.ref_counted(), 0);
                 (o, parent_config)
             }
@@ -394,7 +430,7 @@ async fn handle_build_plan(
                             let resolved_cwd = image_cwd.join(cwd);
                             let resolved_cwd = resolved_cwd.to_str().unwrap(); // TODO: report error if image cwd is not valid utf8.
                             cd(resolved_cwd, &mut script);
-                            for (k, v) in additional_envs.iter() {
+                            for (k, v) in iter_hm_sorted(additional_envs) {
                                 script.push(format!(
                                     "export {}={}",
                                     escape(k.into()),
